/*
 * ROX-MMS - a ROX panel applet front-end to XMMS.
 * By Peter Geer <geerp@cs.sunyit.edu>
 *
 * Adapted from:
 *
 * Demo - a demonstration of coding a ROX applet in C
 *
 * To adapt this as your own program, first replace all occurances of
 * "DemoApplet" or "DEMOAPPLET" with your applet name.
 *
 * Stephen Watson <stephen@kerofin.demon.co.uk>
 *
 * GPL applies.
 *
 * $Id: main.c,v 1.9 2003/07/11 02:39:31 Treize Exp $
 */
#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>

/* These next two are for internationalization */
#ifdef HAVE_LOCALE_H
#include <locale.h>
#endif
#ifdef HAVE_LIBINTL_H
#include <libintl.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <xmms/xmmsctrl.h>
#include "infowin.h"
#include "choices.h"
#include "global.h"
#include "configwin.h"

#include "pixmaps/play_pause.xpm"
#include "pixmaps/stop.xpm"
#include "pixmaps/next.xpm"
#include "pixmaps/prev.xpm"
#include "pixmaps/eject.xpm"
#include "pixmaps/underscore.xpm"

#define DEBUG              1   /* Allow debug output */
#define UPDATE_FREQUENCY 200
/* These are for the label status. */
#define LABEL_TITLE  0
#define LABEL_TIME   1
#define LABEL_MODULO 2

/* It's probably a good idea to have constants for the buttons. */
#define BTN_PLAY_PAUSE 0
#define BTN_STOP       1
#define BTN_PREV       2
#define BTN_NEXT       3
#define BTN_EJECT      4

/* Global data. */
static gint xmms_session;
GtkWidget *label;  /* This name was poorly chosen. */
/* As the timer is always running now, this is no longer needed.
   However, we'll keep it, because we have to assign the return 
	value to something and this makes it easier if we decide later
	that the timer _shouldn't_ always be running. */
guint timer_id;
int label_status=0;

/* Declare functions in advance */
static gint button_press(GtkWidget *window, GdkEventButton *bev,
			 gpointer win); /* button press on window */
static void show_info_win(void);        /* Show information box */
static void read_config(void);          /* Read configuration */
static void write_config(void);         /* Write configuration */

/* Generates (optional) debug messages */
void dprintf(int level, const char *fmt, ...);

/* Utility Functions: */
void add_button(GtkWidget *to_hbox, char *icon[], gint btn_code);
/* Hopefully this will make a pixmap like I think it will. */
GtkWidget *create_pixmap_d(GtkWidget * widget, char *from[]);
gchar *scroll_string(gchar *title);
char *get_time_string(char *format);
void run_xmms(gint session);

/* Here are my call-back function declarations. */
gint clicked(GtkWidget *widget, gpointer data);
gint timer_callback();
static void dnd_callback(GtkWidget *widget, GdkDragContext *dc,
        gint x, gint y, GtkSelectionData *selection_data,  
        guint info, guint t, gpointer data);

/* Main.  Here we set up the gui and enter the main loop */
int main(int argc, char *argv[])
{
  GtkWidget *win=NULL;
  GtkWidget *vbox, *hbox;
  GtkWidget *ev_box;
  GtkWidget *length_fixer;
  GtkTargetEntry targets[3];
  int i;
  
  gchar *app_dir;
  guint32 xid;
#ifdef HAVE_BINDTEXTDOMAIN
  gchar *localedir;
#endif

  /* First things first, set the locale information for time, so that
     strftime will give us a sensible date format... */
#ifdef HAVE_SETLOCALE
  setlocale(LC_TIME, "");
  setlocale (LC_ALL, "");
#endif
  /* What is the directory where our resources are? (set by AppletRun) */
  app_dir=g_getenv("APP_DIR");
#ifdef HAVE_BINDTEXTDOMAIN
  /* More (untested) i18n support */
  localedir=g_strconcat(app_dir, "/Messages", NULL);
  bindtextdomain(PROJECT, localedir);
  textdomain(PROJECT);
  g_free(localedir);
#endif
  /* Set configuration defaults. */
  xmms_session = 0;
  label_status = 0;  
  UpdateFrequency = 200;
  TitleScrollLength = 15;
  PadReduction = 2;
  cdrom_mount_path = g_strdup("/mnt/cdrom");
  time_format = g_strdup("%l:%M %p %Z");
  date_format = g_strdup("%a, %d-%b-%Y");

  targets[0].target = "text/plain";
  targets[0].flags = 0;
  targets[0].info = 0;
  targets[1].target = "text/uri-list";
  targets[1].flags = 0;
  targets[1].info = 1;
  targets[2].target = "STRING";
  targets[2].flags = 0;
  targets[2].info = 2;
  
  /* Initialise X/GDK/GTK */
  gtk_init(&argc, &argv);
  gdk_rgb_init();
  gtk_widget_push_visual(gdk_rgb_get_visual());
  gtk_widget_push_colormap(gdk_rgb_get_cmap());
  
  /* Init choices and read them in */
  choices_init();
  read_config();
  config_win_init();

  /* If we have an argument, try to interperet it as an XID */
  if(argc>1)
    xid=atol(argv[1]);
  else
    xid=0;

  if(xid) {
    /* We are an applet, create a plug to go in the filer's socket */
    GtkWidget *plug;

    dprintf(3, "xid=%lu\n", xid);
    plug=gtk_plug_new(xid);
    if(!plug) {
      fprintf(stderr, _("%s: failed to plug into socket %s, not a XID?\n"),
	      argv[0], argv[1]);
      exit(1);
    }
    gtk_signal_connect(GTK_OBJECT(plug), "destroy", 
		       GTK_SIGNAL_FUNC(gtk_main_quit), 
		       "WM destroy");
    gtk_signal_connect(GTK_OBJECT(plug), "button_press_event",
		       GTK_SIGNAL_FUNC(button_press), plug);
    gtk_widget_add_events(plug, GDK_BUTTON_PRESS_MASK);
    /* Isn't object oriented code wonderful.  Now that we have done the
       plug specific code, its just a containing widget.  For the rest of the
       program we just pretend it was a window and it all works
    */
    win=plug;
  } else { 
    /* Create window */
    win=gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_name(win, "ROX-MMS");
    gtk_window_set_title(GTK_WINDOW(win), _("ROX-MMS"));
    gtk_signal_connect(GTK_OBJECT(win), "destroy", 
		       GTK_SIGNAL_FUNC(gtk_main_quit), 
		       "WM destroy");
  }

  /* win now refers to either a window or the applet, and it doesn't matter
     which... */
  /* We want to pop up a menu on a button press */
  gtk_signal_connect(GTK_OBJECT(win), "button_press_event", GTK_SIGNAL_FUNC(button_press), win);
		     
  gtk_widget_add_events(win, GDK_BUTTON_PRESS_MASK);

  /* A vbox contains widgets arranged vertically */
  vbox=gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(win), vbox);
  gtk_widget_show(vbox);

  ev_box = gtk_event_box_new();
  /* Set up the event box for drag-and-drop file loading.
     This code stolen from the Wolfpack DnD tutorial. 
	  I'm not exactly certain if I need all of this.*/
  gtk_drag_dest_set(ev_box, GTK_DEST_DEFAULT_MOTION | 
       GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP, 
	    targets, sizeof(targets)/sizeof(GtkTargetEntry), 
	    GDK_ACTION_MOVE | GDK_ACTION_COPY);
  gtk_signal_connect(GTK_OBJECT(ev_box), "drag_data_received", GTK_SIGNAL_FUNC(dnd_callback), win);
  label=gtk_label_new("ROX-MMS");
  gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
  gtk_container_add(GTK_CONTAINER(ev_box), label);
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(vbox), ev_box, TRUE, TRUE, 0);
  gtk_widget_set_events(ev_box, GDK_BUTTON_PRESS_MASK);
  gtk_signal_connect(GTK_OBJECT(ev_box), "button_press_event", GTK_SIGNAL_FUNC(button_press), win);
  gtk_signal_connect(GTK_OBJECT(win), "button_press_event", GTK_SIGNAL_FUNC(button_press), win);
  gtk_widget_show(ev_box);
  gtk_widget_realize(ev_box);
  
  /* Let's add a fixed number of pixmaps that are just underscores.
     This should give us the fixed spacing we want. */
  hbox = gtk_hbox_new(TRUE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show(hbox);
  for (i = 1; i <= TitleScrollLength - PadReduction; i++) {
    ev_box = gtk_event_box_new();
    gtk_box_pack_start(GTK_BOX(hbox), ev_box, FALSE, FALSE, 0);
    length_fixer = create_pixmap_d(ev_box, underscore_xpm);
    gtk_widget_show(length_fixer);
    gtk_container_add(GTK_CONTAINER(ev_box), length_fixer);
    gtk_container_set_border_width(GTK_CONTAINER(ev_box), 0);
    gtk_widget_show(ev_box);
  }

  /* Add an hbox for our buttons. */
  hbox=gtk_hbox_new(TRUE, 0); 
  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
  gtk_widget_show(hbox); 

  /* Now it's time to add our buttons. */
  add_button(hbox, play_pause_xpm, BTN_PLAY_PAUSE);
  add_button(hbox, stop_xpm, BTN_STOP);
  add_button(hbox, prev_xpm, BTN_PREV);
  add_button(hbox, next_xpm, BTN_NEXT);
  add_button(hbox, eject_xpm, BTN_EJECT);

  /* In case XMMS is already running, we need to start our timeout. */
  /*if (xmms_remote_is_running(xmms_session)) {*/
  timer_id = gtk_timeout_add(UpdateFrequency, timer_callback, NULL);
  /*}*/

  /* Show our new window/applet */
  gtk_widget_show(win);

  /* Main processing loop */
  gtk_main();

  /* We are done, save out config incase it changed */
  write_config();

  return 0;
}

/* Ad hoc function to add an XMMS control button to the window. */
void add_button(GtkWidget *to_hbox, char *icon[], gint btn_code) {
  
  GtkWidget *button;
  GtkWidget *control_pixmap;
  
  button = gtk_button_new(); 
  gtk_box_pack_start(GTK_BOX(to_hbox), button, TRUE, TRUE, 0);
  control_pixmap = create_pixmap_d(button, icon);
  gtk_widget_show(control_pixmap);
  gtk_container_add(GTK_CONTAINER(button), control_pixmap);
  gtk_container_set_border_width(GTK_CONTAINER(button), 0);
  gtk_widget_show(button);
  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(clicked), GINT_TO_POINTER(btn_code) );
  
}

/*
 * Generates debug output, but only if:
 * DEBUG is defined as non-zero
 * $ROXMMS_DEBUG_LEVEL is greater than 0
 *
 * level is the urgency of the message, in the range 0-5 (roughly).  Only
 * if level<=$ROXMMS_DEBUG_LEVEL will the message be printed
 */
void dprintf(int level, const char *fmt, ...)
{
#if DEBUG
  va_list list;
  static int dlevel=-1;

  if(dlevel==-1) {
    gchar *val=g_getenv("ROXMMS_DEBUG_LEVEL");
    if(val)
      dlevel=atoi(val);
    if(dlevel<0)
      dlevel=0;
  }

  if(level>dlevel)
    return;

  va_start(list, fmt);
  g_logv(PROJECT, G_LOG_LEVEL_DEBUG, fmt, list);
  va_end(list);  
#endif
}


/* Write the config to a file. */
static void write_config(void) {

	gchar *fname;

	/* Use the choices system to get the name to save to */
	fname=choices_find_path_save("options", PROJECT, TRUE);
	dprintf(2, "save to %s", fname? fname: "NULL");

	if(fname) {
		FILE *out;

		out=fopen(fname, "w");
		if(out) {
			time_t now;
			char buf[80];

			fprintf(out, _("# Config file for %s %s (%s)\n"), PROJECT, VERSION, AUTHOR);
			fprintf(out, _("# Latest version at %s\n"), WEBSITE);
			time(&now);
			strftime(buf, 80, "%c", localtime(&now));
			fprintf(out, _("#\n# Written %s\n\n"), buf);

			/* Write config here... */
			fprintf(out, _("update_frequency = %d\n"), UpdateFrequency);
			fprintf(out, _("title_length = %d\n"), TitleScrollLength);
			fprintf(out, _("pad_reduction = %d\n"), PadReduction);
			fprintf(out, _("cd_mount_path = %s\n"), cdrom_mount_path);
			fprintf(out, _("time_format = \"%s\"\n"), time_format);
			fprintf(out, _("date_format = \"%s\"\n"), date_format);
      
			fclose(out);
		}

		g_free(fname);
	}
}

/* Read in the config. */
static void read_config(void) {

	guchar *fname;

	/* Use the choices system to locate the file to read */
	fname=choices_find_path_load("options", PROJECT);

	if(fname) {
		FILE *in;
		in=fopen(fname, "r");
		if(in) {
			char buf[1024], *line;
			char *end, *start;
			int val;

			do {
				line=fgets(buf, sizeof(buf), in);
				if(!line)
					break;
				end=strchr(line, '\n');
				if(end)
					*end=0;
				end=strchr(line, '#');  /* everything after # is a comment */
				if(end)
					*end=0;

				/* Process the line here... */
				if( strstr(line, "update_frequency") ) {
					
					start = strstr(line, "=");
					start = (char*) start+1;
					val = atoi(start);
					if (val > 0) UpdateFrequency = val;

				} else if ( strstr(line, "title_length") ) {

					start = strstr(line, "=");
					start = (char*) start+1;
					val = atoi(start);
					if (val > 0) TitleScrollLength = val;
					
				} else if ( strstr(line, "pad_reduction") ) {

					start = strstr(line, "=");
					start = (char*) start+1;
					val = atoi(start);
					if (val > 0) PadReduction = val;

				} else if ( strstr(line, "cd_mount_path") ) {

					start = strstr(line, "=");
					start = (char*) start+1;
					while (start[0] != '/' && start[0] != '\0')
						start = (char*) start+1;
					
					/*g_free(cdrom_mount_path);*/
					cdrom_mount_path = g_strdup(start);
					
				} else if ( strstr(line, "time_format") ) {
					
					start = strstr(line, "=");
					while (start[0] != '"' && start[0] != '\0')
						start = (char*) start+1;

					if (start[0] != '\0') {
						/* Go to the first character after the quote. */
						start = (char*) start+1;
						/* Find the closing quote and end the string there. */
						end = strstr(start, "\"");
						if (end != start) end[0] = '\0';
						time_format = g_strdup(start);
					}
					
				} else if ( strstr(line, "date_format") ) {
					
					start = strstr(line, "=");
					while (start[0] != '"' && start[0] != '\0')
						start = (char*) start+1;

					if (start[0] != '\0') {
						/* Go to the first character after the quote. */
						start = (char*) start+1;
						/* Find the closing quote and end the string there. */
						end = strstr(start, "\"");
						if (end != start) end[0] = '\0';
						date_format = g_strdup(start);
					}
				}

			} while(!feof(in));

			fclose(in);
		}

		g_free(fname);
	}
}

/* Check if XMMS is running.  If it's not, try to start it and wait 
   until it's running to return. */
void run_xmms (gint_session) {
	if (!xmms_remote_is_running(xmms_session)) {
		/* There's probably a better way to start XMMS, but I don't know
		   enough about C to know what it is.*/
		system("xmms &");
		while ( ! xmms_remote_is_running(xmms_session) ) {
			sleep(1);
		}
	}
}

static void play_audio_cd() {
	run_xmms(xmms_session);
	xmms_remote_playlist_clear(xmms_session);
	xmms_remote_playlist_add_url_string(xmms_session, cdrom_mount_path);
	xmms_remote_set_playlist_pos(xmms_session, 0);
	xmms_remote_play(xmms_session);
}

static void quit_xmms() {
	xmms_remote_quit(xmms_session);
}

static void clear_playlist() {
	run_xmms(xmms_session);
	xmms_remote_playlist_clear(xmms_session);
}

/*
 * Pop-up menu
 */
static GtkItemFactoryEntry menu_items[] = {
	{ N_("/Audio CD"), NULL, play_audio_cd, 0, NULL},
	{ N_("/Clear Playlist"), NULL, clear_playlist, 0, NULL},
	{ N_("/Quit XMMS"), NULL, quit_xmms, 0, NULL }, 
	{ N_("/sep"), NULL, NULL, 0, "<Separator>" },
	{ N_("/Configure"), NULL, config_win_show, 0, NULL},
	{ N_("/Info"), NULL, show_info_win, 0, NULL },
	{ N_("/Quit"), NULL, gtk_main_quit, 0, NULL },
};

/* Save user-defined menu accelerators */
static void save_menus(void)
{
  char	*menurc;
	
  menurc = choices_find_path_save("menus", PROJECT, TRUE);
  if (menurc) {
    gtk_item_factory_dump_rc(menurc, NULL, TRUE);
    g_free(menurc);
  }
}

/* Create the pop-up menu */
static GtkWidget *menu_create_menu(GtkWidget *window)
{
  GtkWidget *menu;
  GtkItemFactory	*item_factory;
  GtkAccelGroup	*accel_group;
  gint 		n_items = sizeof(menu_items) / sizeof(*menu_items);
  gchar *menurc;

  accel_group = gtk_accel_group_new();

  item_factory = gtk_item_factory_new(GTK_TYPE_MENU, "<system>", 
				      accel_group);

  gtk_item_factory_create_items(item_factory, n_items, menu_items, NULL);

  /* Attach the new accelerator group to the window. */
  gtk_accel_group_attach(accel_group, GTK_OBJECT(window));

  menu = gtk_item_factory_get_widget(item_factory, "<system>");

  /* Load any user-defined menu accelerators */
  menurc=choices_find_path_load("menus", PROJECT);
  if(menurc) {
    gtk_item_factory_parse_rc(menurc);
    g_free(menurc);
  }

  /* Save updated accelerators when we exit */
  atexit(save_menus);

  return menu;
}

/* Button press on our window/applet */
static gint button_press(GtkWidget *window, GdkEventButton *bev,
			 gpointer win)
{
	static GtkWidget *menu=NULL;
	
	/*  Right-clicking pops up the context menu. */
	if(bev->type==GDK_BUTTON_PRESS && bev->button==3) {
    /* Pop up the menu */
		if(!menu) 
			menu=menu_create_menu(GTK_WIDGET(win));
    	
		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
						bev->button, bev->time);
		return TRUE;
	/* Left-clicking toggles the display. */
	} else if (bev->type == GDK_BUTTON_PRESS && bev->button == 1) {
		label_status = (label_status+1) % LABEL_MODULO;
		return TRUE;
	}
	return FALSE;
}

/* Make a destroy-frame into a close, allowing us to re-use the window */
static int trap_frame_destroy(GtkWidget *widget, GdkEvent *event,
			      gpointer data)
{
  /* Change this destroy into a hide */
  gtk_widget_hide(GTK_WIDGET(data));
  return TRUE;
}

/* Show the info window */
static void show_info_win(void)
{
  static GtkWidget *infowin=NULL;
  
  if(!infowin) {
    /* Need to make it first.  The arguments are macros defined
     in config.h.in */
    infowin=info_win_new(PROJECT, PURPOSE, VERSION, AUTHOR, WEBSITE);
    gtk_signal_connect(GTK_OBJECT(infowin), "delete_event", 
		     GTK_SIGNAL_FUNC(trap_frame_destroy), 
		     infowin);
  }

  gtk_widget_show(infowin);
}

/* This function stolen from the ROX-Menu file menu_applet.c,
   by Leandro Pereira. */
GtkWidget *create_pixmap_d(GtkWidget * widget, char *from[])
{
    GdkColormap *colormap;
    GdkPixmap *gdkpixmap;
    GdkBitmap *mask;
    GtkWidget *pixmap;

    colormap = gtk_widget_get_colormap(widget);
    gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask,
						      NULL, from);
    pixmap = gtk_pixmap_new(gdkpixmap, mask);
    gdk_pixmap_unref(gdkpixmap);
    gdk_bitmap_unref(mask);

    return pixmap;
}

/* Here we take care of our button presses. */
gint clicked(GtkWidget *widget, gpointer data) {
	int option;
	option = GPOINTER_TO_INT(data);
	run_xmms(xmms_session);
	switch(option) {
		case BTN_PLAY_PAUSE:
			xmms_remote_play_pause(xmms_session);
			break;
		case BTN_STOP:
			xmms_remote_stop(xmms_session);
			break;
		case BTN_PREV:
			xmms_remote_playlist_prev(xmms_session);
			break;
		case BTN_NEXT:
			xmms_remote_playlist_next(xmms_session);
			break;
		case BTN_EJECT:
			xmms_remote_eject(xmms_session);
			break;
	}
	return 0;
}

gint timer_callback() {

	/* For displaying the date and time. */
	char* time_string = NULL;

	gint pos = -1;
	gint list_len;
	static gchar *title_string;
	static gchar *current_title=NULL;
	gchar **label_string_ptr;
	gchar *temp=NULL;
	gchar *cap_string = NULL;
	int len;
	gint time, t_min, t_sec, track_length, l_min, l_sec;
	gboolean skip_scroll = FALSE;
	
	if(xmms_remote_is_running(xmms_session)) {
		list_len = xmms_remote_get_playlist_length(xmms_session);
		if (list_len > 0) {
			pos = xmms_remote_get_playlist_pos(xmms_session);
			temp = xmms_remote_get_playlist_title(xmms_session, pos);
		} else {
			pos = -1;
			temp = NULL;
		}
		if (label_status == LABEL_TITLE && pos >= 0) {
			if (temp == NULL || list_len == 0) {
				
				gtk_label_set_text(GTK_LABEL(label), "No track...");
				g_free(current_title);
				g_free(title_string);
				g_free(temp);
				pos = -1;
				return TRUE;
			
			} else if (current_title == NULL || g_strcasecmp(temp, current_title) != 0) {
			
				g_free(title_string);
				g_free(current_title);
				current_title = temp;
				temp = NULL;
				title_string = g_strdup(current_title);
				len = strlen(title_string);
				
				if (len >= TitleScrollLength) {
					temp = g_strjoin(NULL, " * ", title_string, NULL);
					g_free(title_string);
					title_string = temp;
					temp = NULL;
				}

			} else {
				g_free(temp);
			}
			
			len = strlen(title_string);
			if (len < TitleScrollLength) skip_scroll = TRUE;
			if (!skip_scroll) scroll_string(title_string);
			g_free(cap_string);
			cap_string = g_strndup(title_string, TitleScrollLength);
			gtk_label_set_text(GTK_LABEL(label), cap_string);
			g_free(cap_string);
			return TRUE;

		} else if (label_status == LABEL_TIME && pos >= 0) {

			time = xmms_remote_get_output_time(xmms_session);
			track_length = xmms_remote_get_playlist_time(xmms_session, pos);
			/* The time appears to be in milliseconds, so we'll convert to straight seconds. */
			t_min = (time - (time % 60000)) / 60000;
			t_sec = (time % 60000) / 1000;
			l_min = (track_length - (track_length % 60000)) / 60000;
			l_sec = (track_length % 60000) / 1000;
			/* Now we can set the caption. */
			g_free(cap_string);
			cap_string = malloc(20);
			sprintf(cap_string, "%d:%02d / %d:%02d", t_min, t_sec, l_min, l_sec);
			gtk_label_set_text(GTK_LABEL(label), cap_string);
			g_free(cap_string);
		
		} else {
			gtk_label_set_text(GTK_LABEL(label), "No track...");
		}
	} else {
		free(time_string);
		if (label_status == LABEL_TIME) 
			time_string = get_time_string(date_format);
		else
			time_string = get_time_string(time_format);
		gtk_label_set_text(GTK_LABEL(label), time_string);
		free(time_string);
	}
	return TRUE;
}

/* This function will "scroll" the string by one character.  In other words, it will 
   shift each character left by one and move the first to the end. */
gchar *scroll_string(gchar *title) {

	char first;
	int i, len;

	first = title[0];
	len = strlen(title);
	memmove(title, (char*)title+1, len-1);
	
	title[len-1] = first;
	return title;

}

/* Return a string for the current time. */
char *get_time_string(char *format) {
	time_t ct;
	struct tm *ctm;
	char *time_string=NULL;
	int size;
	/* Can this possibly be less than the actual size? */
	size = strlen(format) * 3;
	time_string=malloc( sizeof(char) * size );
	time(&ct);
	ctm = localtime(&ct);
	strftime(time_string, size, format, ctm); /*"%l:%M %p %Z", ctm);*/
	if (time_string == NULL) return NULL;
	free(ctm);
	return time_string;
}

static void dnd_callback(GtkWidget *widget, GdkDragContext *dc,
	gint x, gint y, GtkSelectionData *selection_data, guint info, 
	guint t, gpointer data) {
	
	if (selection_data != NULL) {
		if(selection_data->length > 0) {
			run_xmms(xmms_session);
			xmms_remote_playlist_add_url_string(xmms_session, selection_data->data);
		}
	}
}
